package com.dgtly.sync.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.dgtly.common.exception.BusinessException;
import com.dgtly.common.utils.ShiroSaltUtil;
import com.dgtly.common.utils.security.EncryptPassWordClass;
import com.dgtly.sync.controller.SyncController;
import com.dgtly.sync.domain.MetaNipponOrgRelation;
import com.dgtly.sync.domain.MetaUserinfoAll;
import com.dgtly.sync.domain.SyncFailDetail;
import com.dgtly.sync.domain.SyncLog;
import com.dgtly.sync.exception.EndLessLoopParentException;
import com.dgtly.sync.exception.NotFoundParentException;
import com.dgtly.sync.mapper.MetaUserinfoAllMapper;
import com.dgtly.sync.mapper.SyncFailDetailMapper;
import com.dgtly.sync.service.IAnalysisDeptService;
import com.dgtly.sync.service.IAnalysisSysUserService;
import com.dgtly.sync.service.IMetaNipponOrgRelationService;
import com.dgtly.sync.service.ISyncLogService;
import com.dgtly.system.domain.SysConfig;
import com.dgtly.system.domain.SysDept;
import com.dgtly.system.domain.SysUser;
import com.dgtly.system.mapper.SysDeptMapper;
import com.dgtly.system.service.ISysConfigService;
import com.dgtly.system.service.ISysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class AnalysisSysUserService implements IAnalysisSysUserService {

    private static final Logger log = LoggerFactory.getLogger(SyncController.class);




    @Autowired
    private SyncFailDetailMapper syncFailDetailMapper;
    @Autowired
    private ISysConfigService configService;

    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private MetaUserinfoAllMapper metaUserinfoAllMapper;
    @Autowired
    private ISyncLogService syncLogService;



    /**
     * 解析部门数据  异步执行
     * @param packageId
     * @return
     */
    @Override
    @Async
    public SyncLog analysisSysUser(String packageId){
        MetaUserinfoAll param = new MetaUserinfoAll();
        param.setPackageId(packageId);

        SyncLog syncLog = new SyncLog();
        syncLog.setSyncType("sysuser");
        syncLog.setStartTime(new Date());

        Set<String> loginNameList = sysUserService.selectAllUserLoginName();
        List<MetaUserinfoAll> metaUserinfoAlls = metaUserinfoAllMapper.selectMetaUserinfoAllList(param);


        syncLog.setPackageId(packageId);
        syncLog.setTotalCount((long)metaUserinfoAlls.size());
        List<SyncFailDetail> failDetails = new ArrayList<>();
        long successCount = 0;
        long failCount = 0;

        Long deptId = Long.parseLong(configService.selectConfigByKey("libang.user.deptid"));
        Long roleId = Long.parseLong(configService.selectConfigByKey("libang.user.roleid"));
        Long[] roleIds = {roleId};
        if(metaUserinfoAlls.size()>0){
            for(MetaUserinfoAll m:metaUserinfoAlls) {
                try {
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(m.getLoginname());
                    user.setUserName(m.getName());
                    user.setEmail(m.getEmail());
                    user.setPhonenumber(m.getMobile());

                    user.setRoleIds(roleIds);
                    if (m.getStatus() == 0) {
                        user.setStatus("1");
                        user.setDelFlag("2");
                    } else {
                        user.setStatus("0");
                    }

                    if (loginNameList.contains(user.getLoginName())) {
                        sysUserService.updateUserInfo(user);
                    } else {
                        //修改密码并加密
                        user.setPassword(m.getLoginname());
                        user.setSalt(ShiroSaltUtil.randomSalt());
                        user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));

                        sysUserService.insertUser(user);
                    }
                    successCount++;
                } catch (Exception e) {
                    failCount++;
                    SyncFailDetail syncFailDetail = new SyncFailDetail();
                    syncFailDetail.setPackageId(packageId);
                    syncFailDetail.setFailReason(e.getMessage());
                    syncFailDetail.setDataJson(JSONObject.toJSONString(m));
                    failDetails.add(syncFailDetail);
                    e.printStackTrace();
                    log.error("解析用户出错", e);
                }
            }
            syncLog.setSuccessCount(successCount);
            syncLog.setFaileCount(failCount);
        }else{
            syncLog.setSuccessCount(0l);
            syncLog.setFaileCount(0l);
        }
        try{
            syncLog.setEndTime(new Date());
            if(failDetails.size()>0){
                syncFailDetailMapper.batchInsertSyncFailDetail(failDetails);
            }
        }catch (Exception e){
            log.error("解析用户插入解析日志出错",e);
            e.printStackTrace();
            throw new BusinessException("解析用户插入解析日志出错",e);
        }finally {
            syncLogService.insertSyncLog(syncLog);
        }
        return syncLog;

    }



}
